// Includes
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <oleauto.h>
#include "vsmsgw.h"

//-----------------------------------------------------------------------------
// Callback wrappers
//-----------------------------------------------------------------------------
typedef int(__stdcall *cbWfuncText)(int HisNodeID, BSTR str, void *arg);
typedef int(__stdcall *cbWfuncPDU)(int HisNodeID, SAFEARRAY **data, int length, void *arg);
typedef int(__stdcall *cbWfuncPacksge)(int HisNodeID, SAFEARRAY **data, int length, void *arg);

cbWfuncText    TextCB_    = 0;
cbWfuncPDU     PduCB_     = 0;
cbWfuncPacksge PackageCB_ = 0;

char *strPos = "[TextCB]point";
int _cdecl PduCB(int HisNodeID, unsigned char *data, int length, void *arg);

int _cdecl TextCB(int HisNodeID, char *str, void *arg)
{
  // VB Callback
  if (TextCB_) {
    // Var
    WCHAR ts[170];
    BSTR  bstr;
    int   rc;

    // Convert from ANSI c string to VB string
    rc = MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, str, -1, ts, 170);
    if (!rc) return 1;
    bstr = SysAllocString(ts);

    // Call VB callback function
    rc = TextCB_(HisNodeID, bstr, arg);

    // Release VB string
    SysFreeString(bstr);

    // Completed
    return rc;
  }

  // Completed
  return 1;
}

int _cdecl PduCB(int HisNodeID, unsigned char *data, int length, void *arg)
{
  // VB Callback
  if (PduCB_) {
    // Var
    BYTE HUGEP *pdat;
    SAFEARRAY  *arDat;

    // Create VB Array
    arDat = SafeArrayCreateVector(VT_UI1, 0, length);
    if (arDat) {
      // Get pointer to data
      if (SafeArrayAccessData(arDat, (void HUGEP**)&pdat) == S_OK) {
        // Copy data
        memcpy(pdat, data, length);
        if (SafeArrayUnaccessData(arDat) == S_OK) {
          // Call VB callback function
          int res = PduCB_(HisNodeID, &arDat, length, arg);

          // Release VB Array
          SafeArrayDestroy(arDat);

          // Completed
          return res;
        }
      }

      // Release VB Array
      SafeArrayDestroy(arDat);
    }
  }

  // Completed
  return 1;
}

int _cdecl PackageCB(int HisNodeID, unsigned char *data, int length, void *arg)
{
  // VB Callback
  if (PackageCB_) {
    // Var
    BYTE HUGEP *pdat;
    SAFEARRAY  *arDat;

    // Create VB Array
    arDat = SafeArrayCreateVector(VT_UI1, 0, length);
    if (arDat) {
      // Get pointer to data
      if (SafeArrayAccessData(arDat, (void HUGEP**)&pdat) == S_OK) {
        // Copy data
        memcpy(pdat, data, length);
        if (SafeArrayUnaccessData(arDat) == S_OK) {
          // Call VB callback function
          int res = PackageCB_(HisNodeID, &arDat, length, arg);

          // Release VB Array
          SafeArrayDestroy(arDat);

          // Completed
          return res;
        }
      }

      // Release VB Array
      SafeArrayDestroy(arDat);
    }
  }

  // Completed
  return 1;
}

//-----------------------------------------------------------------------------
// Interface
//-----------------------------------------------------------------------------

int __stdcall vsmsgwInit_(int MyNodeID, const char *GWIP, int GWPort, const char *GWKey, cbWfuncText SMSText, cbWfuncPDU SMSPDU, void *arg)
{
  // Callback Wrappers
  TextCB_ = SMSText;
  PduCB_  = SMSPDU;
  // Call function
  return vsmsgwInit(MyNodeID, GWIP, GWPort, GWKey, TextCB, PduCB, arg);
}

int __stdcall vsmsgwInitAdv_(int MyNodeID, const char *GWIP, int GWPort, const char *GWKey, unsigned char *CryptKey)
{
  return vsmsgwInitAdv(MyNodeID, GWIP, GWPort, GWKey, CryptKey);
}

void __stdcall vsmsgwSetSMSTextCB_(cbWfuncText SMSText, void *arg)
{
  // Callback Wrappers
  TextCB_ = SMSText;
  // Call function
  return vsmsgwSetSMSTextCB(TextCB, arg);
}

void __stdcall vsmsgwSetSMSPDUCB_(cbWfuncPDU SMSPDU, void *arg)
{
  // Callback Wrappers
  PduCB_ = SMSPDU;
  // Call function
  return vsmsgwSetSMSPDUCB(PduCB, arg);
}

void __stdcall vsmsgwSetDataPackageCB_(cbWfuncPacksge PACKAGE, void *arg)
{
  // Callback Wrappers
  PackageCB_ = PACKAGE;
  // Call function
  return vsmsgwSetDataPackageCB(PackageCB, arg);
}

bool __stdcall vsmsgwIsConnected_(void)
{
  return vsmsgwIsConnected();
}

int __stdcall vsmsgwGetMyNodeID_(unsigned long* MyNodeID)
{
  return vsmsgwGetMyNodeID(MyNodeID);
}

int __stdcall vsmsgwSendSMS_(unsigned int HisNodeID, const char *str, int *rc)
{
  return vsmsgwSendSMS(HisNodeID,str,rc);
}

int __stdcall vsmsgwSendPDU_(unsigned int HisNodeID, unsigned char *data, int length, int *rc)
{
  return vsmsgwSendPDU(HisNodeID, data, length, rc);
}

int __stdcall vsmsgwSendDataPackage_(unsigned int HisNodeID, unsigned char *data, int length, int *rc)
{
  return vsmsgwSendDataPackage(HisNodeID, data, length, rc);
}

//-----------------------------------------------------------------------------
// Dll entry
//-----------------------------------------------------------------------------

BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved )
{
  return TRUE;
}
